/*
 * Example 41-8
 * A canonical Tk main program and Tk_AppInit.
 */

/* main.c */
#include <tk.h>
#include <Dbg.h>
#include <signal.h>

int Tk_AppInit(Tcl_Interp *interp);

/*
* Handler procedure for async event. See page 542.
*/
void Sig_Setup();
RETSIGTYPE Sig_HandleINT(int sig);
int Sig_HandleSafe(ClientData data, 
		Tcl_Interp *interp, int code);
/*
 * Needed if entering the debugger from the idle loop.
 */
static Tcl_Interp *myInterp;

/*
 * A table for command line arguments.
 */
char *myoption1 = NULL;
static int debug = 0;

static Tk_ArgvInfo argTable[] = {
	{"-myoption1", TK_ARGV_STRING, (char *) NULL,
	(char *) &myoption1, "Explain myoption1"},
	{"-debug", TK_ARGV_CONSTANT, (char *) 1, (char *) &debug,
		"Enable the Tcl debugger"},
	{"", TK_ARGV_END, },
};

main(int argc, char *argv[]) {
	Tcl_Interp *interp;
	/*
	 * Save a copy of the command line args for debugger.
	 */
	argv = Dbg_ArgcArgv(argc, argv, 1);
	/*
	 * Create an interpreter for the error message from
	 * Tk_ParseArgv. Another one is created by Tk_Main.
	 * Parse our arguments and leave the rest to Tk_Main.
	 */
	interp = Tcl_CreateInterp();
	if (Tk_ParseArgv(interp, (Tk_Window) NULL, &argc, argv,
			argTable, 0) != TCL_OK) {
		fprintf(stderr, "%s\n", interp->result);
		exit(1);
	}
	Tcl_DeleteInterp(interp);
	/*
	 * Set up a signal handler so we can enter the Tcl 
	 * debugger.
	 */
	Sig_Setup();

	Tk_Main(argc, argv, Tk_AppInit);
	exit(0);
}

int ClockCmd(ClientData clientData,
				Tcl_Interp *interp,
				int argc, char *argv[]);

/*
 * Our pixmap image type.
 */
extern Tk_ImageType tkPixmapImageType;

int
Tk_AppInit(Tcl_Interp *interp) {
	/*
	 * Initialize packages
	 */
	if (Tcl_Init(interp) == TCL_ERROR) {
		return TCL_ERROR;
	}
	if (Tk_Init(interp) == TCL_ERROR) {
		return TCL_ERROR;
	}
	if (Dbg_Init(interp) == TCL_ERROR) {
		return TCL_ERROR;
	}
	if (debug) {
	    /*
	     * Enter the Tcl debugger before the first Tcl com-mand.
	     */
	    Dbg_On(interp, 0);
	}
	myInterp = interp;
	/*
	 * Define application-specific commands here.
	 */
	Tcl_CreateCommand(interp, "clock", ClockCmd,
		(ClientData)Tk_MainWindow(interp),
		(Tcl_CmdDeleteProc *)NULL);

	Tk_CreateImageType(&tkPixmapImageType);
	/*
	 * Define start-up filename. This file is read in
	 * case the program is run interactively.
	 */
	Tcl_SetVar(interp, "tcl_rcFileName", "~/.mytcl",
		TCL_GLOBAL_ONLY);
	return TCL_OK;
}


